@@ -501,37 +501,52 @@ formula_contains_additive_terms <- function(formula) {
 ## @param data The original data frame for the full formula.
 ## @param y The response vector. Default NULL.
 ## @param split_formula If TRUE breaks the response down into single response
-##   formulas.
-## Default FALSE. It only works if `y` represents a multi-output response.
+##   formulas. Default FALSE. It only works if `y` represents a multi-output
+##   response.
+## @param y_unqs Only relevant for augmented-data projection, in which case this
+##   needs to be the vector of unique response values (for an ordinal response,
+##   these need to be ordered correctly, see [extend_family()]'s argument
+##   `augdat_y_unqs`).
 ## @return a list including the updated formula and data
 subset_formula_and_data <- function(formula, terms_, data, y = NULL,
-                                    split_formula = FALSE) {
+                                    split_formula = FALSE, y_unqs = NULL) {
   formula <- make_formula(terms_, formula = formula)
   response_name <- extract_terms_response(formula)$response
 
-  response_cols <- paste0(".", response_name)
+  if (is.null(y_unqs)) {
+    response_cols <- paste0(".", response_name)
 
-  if (NCOL(y) > 1) {
-    response_cols <- paste0(response_cols, ".", seq_len(ncol(y)))
-    if (!split_formula) {
-      response_vector <- paste0(
-        "cbind(",
-        paste(response_cols, collapse = ", "),
-        ")"
-      )
-      formula <- update(formula, paste0(response_vector, " ~ ."))
+    if (NCOL(y) > 1) {
+      response_cols <- paste0(response_cols, ".", seq_len(ncol(y)))
+      if (!split_formula) {
+        response_vector <- paste0(
+          "cbind(",
+          paste(response_cols, collapse = ", "),
+          ")"
+        )
+        formula <- update(formula, paste0(response_vector, " ~ ."))
+      } else {
+        formula <- lapply(response_cols, function(response) {
+          update(formula, paste0(response, " ~ ."))
+        })
+      }
     } else {
-      formula <- lapply(response_cols, function(response) {
-        update(formula, paste0(response, " ~ ."))
-      })
+      formula <- update(formula, paste(response_cols, "~ ."))
     }
+
+    ## don't overwrite original y name
+    data <- data.frame(.z = y, data)
+    colnames(data)[seq_len(NCOL(y))] <- response_cols
   } else {
-    formula <- update(formula, paste(response_cols, "~ ."))
+    # Create the augmented dataset:
+    stopifnot(is.data.frame(data))
+    data <- do.call(rbind, lapply(y_unqs, function(y_unq) {
+      data[[response_name]] <- y_unq
+      return(data)
+    }))
+    data[[response_name]] <- factor(data[[response_name]], levels = y_unqs)
   }
 
-  ## don't overwrite original y name
-  data <- data.frame(.z = y, data)
-  colnames(data)[seq_len(NCOL(y))] <- response_cols
   return(nlist(formula, data))
 }
 
@@ -647,7 +662,8 @@ select_possible_terms_size <- function(chosen, terms, size) {
   valid_submodels <- unlist(valid_submodels[!is.na(valid_submodels)])
   if (length(chosen) > 0) {
     add_chosen <- paste0(" + ", paste(chosen, collapse = "+"))
-    remove_chosen <- paste0(" - ", paste(chosen, collapse = "-"))
+    remove_chosen <- paste0(" - ",
+                            paste(gsub("\\+", "-", chosen), collapse = "-"))
   } else {
     add_chosen <- ""
     remove_chosen <- ""
@@ -691,10 +707,38 @@ count_terms_chosen <- function(list_of_terms, duplicates = TRUE,
 }
 
 ## Helper function to evaluate right hand side formulas in a context
+## Caution: This does not evaluate the right-hand side of a *full formula* such
+## as `y ~ x`, but the right-hand side of a *right-hand side formula* such as
+## `~ x`.
 ## @param formula Formula to evaluate.
 ## @param data Data with which to evaluate.
 ## @return output from the evaluation
 eval_rhs <- function(formula, data) {
+  stopifnot(length(formula) == 2)
+  eval_el2(formula = formula, data = data)
+}
+
+# Helper function to evaluate the left-hand side of a `formula` in a specific
+# environment `data`. This refers to full formulas such as `y ~ 1` or `y ~ x`
+# (in R, there is no such thing like a "left-hand side formula").
+#
+# @param formula A `formula` whose left-hand side should be evaluated.
+# @param data Passed to argument `envir` of eval().
+#
+# @return The output from eval().
+eval_lhs <- function(formula, data) {
+  stopifnot(length(formula) == 3)
+  eval_el2(formula = formula, data = data)
+}
+
+# Helper function to evaluate the second element of a `formula` in a specific
+# environment `data`.
+#
+# @param formula A `formula` whose second element should be evaluated.
+# @param data Passed to argument `envir` of eval().
+#
+# @return The output from eval().
+eval_el2 <- function(formula, data) {
   eval(formula[[2]], data, environment(formula))
 }
 
